package com.sumancs.flutterpaytmpayment;

import android.app.Activity;
import android.os.Bundle;

import androidx.annotation.NonNull;

import com.paytm.pgsdk.PaytmOrder;
import com.paytm.pgsdk.PaytmPGService;
import com.paytm.pgsdk.PaytmPaymentTransactionCallback;

import java.util.HashMap;
import java.util.Map;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/**
 * FlutterpaytmpaymentPlugin
 */
public class FlutterpaytmpaymentPlugin implements MethodCallHandler {
    /// The MethodChannel that will the communication between Flutter and native Android
    ///
    /// This local reference serves to register the plugin with the Flutter Engine and unregister it
    /// when the Flutter Engine is detached from the Activity
    private final Activity activity;
    String TAG = getClass().getName();
    Result result;

    // This static function is optional and equivalent to onAttachedToEngine. It supports the old
    // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
    // plugin registration via this function while apps migrate to use the new Android APIs
    // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
    //
    // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
    // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
    // depending on the user's project. onAttachedToEngine or registerWith must both be defined
    // in the same class.
    public static void registerWith(Registrar registrar) {
        final MethodChannel channel = new MethodChannel(registrar.messenger(), "flutterpaytmpayment");
        channel.setMethodCallHandler(new FlutterpaytmpaymentPlugin(registrar.activity()));
    }

    private FlutterpaytmpaymentPlugin(Activity activity) {
        this.activity = activity;
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        this.result = result;
        if (call.method.equals("startPaytmPayment")) {
            boolean testing = call.argument("testing");
            String mId = call.argument("mId").toString();
            String orderId = call.argument("orderId").toString();
            String custId = call.argument("custId").toString();
            String channelId = call.argument("channelId").toString();
            String txnAmount = call.argument("txnAmount").toString();
            String website = call.argument("website").toString();
            String callBackUrl = call.argument("callBackUrl").toString();
            String industryTypeId = call.argument("industryTypeId").toString();
            String checkSumHash = call.argument("checkSumHash").toString();

            initializePaytmPayment(testing, mId, orderId, custId, channelId, txnAmount, website, callBackUrl, industryTypeId, checkSumHash);

        } else {
            result.notImplemented();
        }
    }

    private void sendResponse(Map<String, Object> paramMap) {

        result.success(paramMap);

    }

    private void initializePaytmPayment(boolean testing, String mId,
                                        String orderId,
                                        String custId,
                                        String channelId,
                                        String txnAmount,
                                        String website,
                                        String callBackUrl,
                                        String industryTypeId,
                                        String checkSumHash) {

        PaytmPGService Service;

        if (testing) {
            Service = PaytmPGService.getStagingService();
        } else {
            Service = PaytmPGService.getProductionService();
        }

        HashMap<String, String> paramMap = new HashMap<>();
        paramMap.put("MID", mId);
        paramMap.put("ORDER_ID", orderId);
        paramMap.put("CUST_ID", custId);
        paramMap.put("CHANNEL_ID", channelId);
        paramMap.put("TXN_AMOUNT", txnAmount);
        paramMap.put("WEBSITE", website);
        paramMap.put("CALLBACK_URL", callBackUrl);
        paramMap.put("CHECKSUMHASH", checkSumHash);
        paramMap.put("INDUSTRY_TYPE_ID", industryTypeId);

        PaytmOrder order = new PaytmOrder(paramMap);

        Service.initialize(order, null);

        Service.startPaymentTransaction(activity, true, true, new PaytmPaymentTransactionCallback() {
            @Override
            public void onTransactionResponse(Bundle bundle) {

                Map<String, Object> paramMap = new HashMap<>();

                for (String key : bundle.keySet()) {
                    paramMap.put(key, bundle.getString(key));
                }

                sendResponse(paramMap);
            }

            @Override
            public void networkNotAvailable() {
                Map<String, Object> paramMap = new HashMap<>();

                paramMap.put("error", true);
                paramMap.put("errorMessage", "Network Not Available");

                sendResponse(paramMap);
            }

            @Override
            public void clientAuthenticationFailed(String s) {

                Map<String, Object> paramMap = new HashMap<>();

                paramMap.put("error", true);
                paramMap.put("errorMessage", s);

                sendResponse(paramMap);
            }

            @Override
            public void someUIErrorOccurred(String s) {

                Map<String, Object> paramMap = new HashMap<>();

                paramMap.put("error", true);
                paramMap.put("errorMessage", s);

                sendResponse(paramMap);

            }

            @Override
            public void onErrorLoadingWebPage(int i, String s, String s1) {

                Map<String, Object> paramMap = new HashMap<>();

                paramMap.put("error", true);
                paramMap.put("errorMessage", s + " , " + s1.toString());

                sendResponse(paramMap);
            }

            @Override
            public void onBackPressedCancelTransaction() {

                Map<String, Object> paramMap = new HashMap<>();

                paramMap.put("error", true);
                paramMap.put("errorMessage", "Back Pressed Transaction Cancelled");

                sendResponse(paramMap);

            }

            @Override
            public void onTransactionCancel(String s, Bundle bundle) {

                Map<String, Object> paramMap = new HashMap<>();


                for (String key : bundle.keySet()) {
                    paramMap.put(key, bundle.getString(key));
                }

                paramMap.put("error", true);
                paramMap.put("errorMessage", s);

                sendResponse(paramMap);
            }
        });

    }

}
